/* $Id: guard.h 358 2010-01-16 06:51:37Z oh2gve $
 * 
 * Copyright 2009, 2010 Tapio Aaltonen
 * 
 * 
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/

#ifndef GUARD_H
#define GUARD_H


// own stuff.
#include "packet.h"
#include "aprspacket.h"
#include "connection.h"
#include "cfg.h"
#include "dupefilter.h"
#include "packetlog.h"

// std stuff.
#include <string>
#include <list>
#include <map>
#include <ctime>


/// Each RF connection has it's own guard.
/**
 * Guard filters out invalid and duplicate packets, remembers stations heard
 * from service area and handles filter and limiter for the connection.
 */
class Guard
{
public:
		
	/// Constructor.
	Guard(Connection* myconn, Cfg::ConnectionCfg const& mycfg, unsigned int const& mynum, unsigned int const& concount, DupeFilter& dupefilter, Packetlog& packetlog, std::string const& owncall);
	
	/// Destructor.
	~Guard();
	
	/// Give a packet, or timeout if there's no incoming packet.
	/**
	 * \param timeout Amount of milliseconds to wait for a packet to appear.
	 */
	Packet* givePacket(unsigned int const& timeout);
	
	/// Checks given packet against our output filter.
	/**
	 * \return True if passes filter, false if not or no filter is defined.
	 */
	bool passesFilter(AprsPacket const* packet) const;
	
	/// Puts given packet in hold to be sent out.
	void sendPacket(Packet* packet);
	
	/// Tells the guard to flush hold_ and clients_.
	void kick();
	  
private:
	
	/// Connection to guard.
	Connection* myconn_;
	
	/// Config of my connection.
	Cfg::ConnectionCfg const& mycfg_;
	
	/// Number of my connection in config file.
	char const mynum_;
	
	/// Template for packet->outcons.
	std::string const outcons_;
	
	/// Duplicate filter.
	DupeFilter& dupefilter_;
	
	/// Packet log.
	Packetlog& packetlog_;
	
	/// Callsign of this IGate.
	std::string const owncall_;
	
	/// Queue of outgoing packets.
	std::list<Packet const*> hold_;
	
	// List of clients who pass our filter and have been heard via this port.
	std::map<std::string, time_t> clients_;
	
	/// Packets sent since last timer reset.
	unsigned int packetCount_;
	
	/// Limiter timer.
	time_t timer_;
};

#endif // GUARD_H
