/* $Id: serial.cpp 358 2010-01-16 06:51:37Z oh2gve $
 * 
 * Copyright 2007, 2008, 2009, 2010 Tapio Aaltonen
 * 
 * 
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/

// own stuff.
#include "cfg.h"
#include "serial.h"

// std stuff.
#include <string>
using std::string;

// wx stuff.
#include <wx/log.h>
#include <wx/utils.h>


Serial::Serial(wxString const& deviceFile, wxString const& lockDir, Cfg::ConnectionState const& state, Cfg::SerialCfg const& portcfg):
Connection(state),
deviceFile_(deviceFile), device_(NULL)
{
	// First we need to find out the name of our port file without any dirs.
	string df (deviceFile.ToAscii());
	string::size_type pos = df.find_last_of("/");
	if ( pos != string::npos )
	{
		string lockFileName(lockDir.ToAscii());
		lockFileName += "/LCK.." + df.substr(pos+1);
	
		device_ = new Serialport(string(deviceFile.ToAscii()), lockFileName, portcfg.speed, portcfg.charlength, portcfg.parity, portcfg.stopbits, portcfg.rtscts);
		
		// Check how it dit.
		if ( !(*device_) )
		{
			wxLogMessage(wxT(" serial(") + deviceFile + wxT("): failed to open device (") + wxString::FromAscii(device_->error().c_str()) + wxT(")"));
			delete device_;
			device_ = NULL;
		}
		else
		{
			// Successfully opened, change our status.
			gotConnected();
			wxLogMessage(wxT(" serial(") + deviceFile + wxT("): device opened."));
		}
	}
	else
	{
		wxLogMessage(wxT(" serial(") + deviceFile + wxT("): the device name is invalid."));
	}
}

Serial::~Serial()
{
	// Close the device.
	delete device_;
}


// PROTECTED


bool Serial::readDevice(string& data)
{
	// Check for existance.
	if ( !device_ )
	{
		wxLogMessage(wxT(" serial(") + deviceFile_ + wxT("): can't read data, device is not open."));
		return false;
	}
	
	// Check status.
	if ( !(*device_) )
	{
		wxLogMessage( wxT(" serial(") + deviceFile_ + wxT("): can't read data, ") + wxString::FromAscii(device_->error().c_str()) );
		delete device_;
		device_ = NULL;
		return false;
	}
	
	// Try to read data.
	if ( !device_->receive(data) )
	{
		wxLogMessage( wxT(" serial(") + deviceFile_ + wxT("): reading failed, ") + wxString::FromAscii(device_->error().c_str()) );
		delete device_;
		device_ = NULL;
		return false;
	}
	
	return true;
}

bool Serial::writeToDevice(const std::string& data)
{
	// Check for existance.
	if ( !device_ )
	{
		wxLogMessage(wxT(" serial(") + deviceFile_ + wxT("): can't write data, device is not open."));
		return false;
	}
	
	// Check status.
	if ( !(*device_) )
	{
		wxLogMessage( wxT(" serial(") + deviceFile_ + wxT("): can't write data, ") + wxString::FromAscii(device_->error().c_str()) );
		delete device_;
		device_ = NULL;
		return false;
	}
	
	// Try to write data.
	if ( !device_->transmit(data) )
	{
		wxLogMessage( wxT(" serial(") + deviceFile_ + wxT("): writing failed, ") + wxString::FromAscii(device_->error().c_str()) );
		delete device_;
		device_ = NULL;
		return false;
	}
	
	return true;
}



bool Serial::portIsConnected() const
{
	return device_ != NULL;
}
