/* $Id: tnc.cpp 358 2010-01-16 06:51:37Z oh2gve $
 * 
 * Copyright 2007, 2008, 2009, 2010 Tapio Aaltonen
 * 
 * 
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/

// own stuff.
#include "cfg.h"
using Cfg::ConnectionState;
#include "aprspacket.h"
#include "tnc.h"

// std stuff.
#include <string>
using std::string;
#include <vector>
using std::vector;
#include <bitset>
using std::bitset;
#include <sstream>
using std::ostringstream;

// wx stuff.
#include <wx/log.h>
#include <wx/utils.h>


// Constructor opens serial port by calling constructor of Serial.
Tnc::Tnc(wxString const& device, Cfg::ConnectionState const& state, Cfg::SerialCfg portcfg, wxString const& lockdir):
Serial(device, lockdir, state, portcfg),
device_(device), inputBuffer_(), tnc2Packet_()
{
}

// PUBLIC MEMBERS

// This is what Tnc thread keeps doing.
void* Tnc::Entry()
{
	threadStarted();
	
	// This allows main thread to continue with waitForConnect().
	startupDone();
	
	// We start here when main thread says ->Run().
	while ( portIsConnected() && !TestDestroy())
	{
		// Check for incoming bytes, save them if necessary.
		if ( inputEnabled() )
		{
			string tmpdata;
			if ( !readDevice(tmpdata) )
			{
				wxLogMessage(wxT(" serialtnc(") + device_ + wxT("): closing device."));
				break;
			}
			inputBuffer_ += tmpdata;
		}
		else
		{
			if ( !readDevice(inputBuffer_) )
			{
				wxLogMessage(wxT(" serialtnc(") + device_ + wxT("): Failed to flush device's input buffer, closing device."));
				break;
			}
			inputBuffer_.clear();
		}
		
		if ( inputBuffer_.size() > 0 )
		{
			// Check for packet.
			if ( findTnc2Packet() )
			{
				// Try to parse the received packet.
				AprsPacket* packet = AprsPacket::fromTnc2Packet(tnc2Packet_);
				inputPush( packet );
				
				// Empty tmp variable for next packet.
				tnc2Packet_.clear();
			}
			else
			{
				// Save orphaned bytes from input buffer, if there's a lot of them.
				if ( inputBuffer_.size() > 256 && inputBuffer_.find("\r\n") == string::npos )
				{
					inputBuffer_.insert(inputBuffer_.end(), '\r');
					inputBuffer_.insert(inputBuffer_.end(), '\n');
					wxLogMessage(wxT(" serialtnc(") + device_ + wxT(") debug: added 2 FENDs to save orphaned bytes."));
				}
			}
		}
		
		// Then let's see if we've got outgoing packets.
		AprsPacket const* incoming = outputPopWait(100);
		if ( incoming )
		{
			// Not implemented, bad idea.
			delete incoming;
		}
	}
	
	// Tell main that we're going down.
	gotDisconnected();
		
	//Wait for main thread to Delete() us, or go straight through if it already did.
	while (true)
	{
		if ( TestDestroy() )
		{
			break;
		}
		
		//If we didn't die yet, wait for 1 ms before checking again.
		wxMilliSleep(1);
	}
	
	return NULL;
}


bool Tnc::findTnc2Packet()
{
	unsigned int packetSize = inputBuffer_.find("\r\n");
	if ( packetSize != string::npos )
	{
		// The \r\n is not included.
		tnc2Packet_ += inputBuffer_.substr(0, packetSize);
		
		//Remove found packet from input buffer. Now \r\n is included.
		packetSize += 2;
		if ( inputBuffer_.size() > packetSize )
		{
			inputBuffer_ = inputBuffer_.substr(packetSize);
		}
		else
		{
			inputBuffer_.clear();
		}
		
		return true;
	}

	return false;
}
